# KarrierWave

TODO: Write a gem description

## Installation

Add this line to your application's Gemfile:

    source 'https://packagecloud.io/ultracaption/rubygems'
    gem 'karrierwave'

And then execute:

    $ bundle


## Usage

아래와 같이 initializer 파일을 만들어 환경에 따라 알맞은 KAGE host를 세팅해준다.

```ruby
# config/initializers/kage.rb

KarrierWave.config do |config|
  if Rails.env.development?
    config.host = KarrierWave::Client::SANDBOX_HOST
  end

  config.http_proxy = 'http://PROXY-URL'
end
```

그리고 KAGE를 사용할 모델에서 아래와 같이 `mount_kage`라는 DSL을 이용해 KAGE를 마운트한다.
첫 번째 인자로 마운트하는 필드명, 두 번째 인자로 기타 옵션을 넣어준다.

```ruby
class PhotoAttachment < ActiveRecord::Base
  mount_kage :photo, config_name: 'klog-main-img',
                     mount_on: :media,
                     snapshot_filename: 'img_xl.jpg',
                     version_keys: {
                       large: '640x*',
                       medium: '320x*',
                       thumb: '168x168'
                     }
end

class VideoAttachment < ActiveRecord::Base
  mount_kage :video, config_name: 'klog-video',
                     filename: 'klogv_720.mp4',
                     snapshot_filename: 'image0.jpg',
                     type: :video,
                     mount_on: :media
end
```

### `mount_kage` 옵션 명세

* `config_name`: KAGE에 설정된 config name.
  이 config name별로 미리 정의된 업로드된 이미지들의 version들이 생성된다.
  엄밀한 의미는 다르지만 S3의 버킷과 비슷한 개념이라고 생각할 수 있다.
* `mount_on`: mount할 이름과 meta data가 저장될 필드명을 다르게 하고 싶으면
  이 옵션으로 실제 serialize되어 저장할 필드명을 지정할 수 있다.
* `filename`: KAGE에 업로드 되는 파일명이 항상 일정한 경우 이 옵션을 이용해 info나 status를 폴링하지 않고 바로 저장하도록 할 수 있다.
* `snapshot_filename`: 동영상이나 animated gif의 snapshot 이미지가 항상 일정한 파일명으로 생성될 경우 이 옵션을 이용해 `snapshot_url`을 생성할 수 있다.
* `type`: 해당 mounter에 업로드되는 미디어의 종류. `:image`, `:animated`(gif), `:video`가 있으며, 기본값은 :image이다. 동영상을 위한 mounter가 아닌 경우는 지정할 필요가 없다.
* `version_keys`: KAGE에는 업로드된 이미지를 동적으로 리사이즈하여 렌더링하는 기능이 있다(역시 미리 정의된 경우).
  위 예시에서는 KAGE 상에 `640x*`라는 리사이즈값이 미리 정의돼 있으나,
  레일즈 어플리케이션에서 `:large`라는 심볼로 부를 수 있도록 하였다.

```ruby
  > p = PhotoAttachment.new
  > p.photo = File.open('/PATH/TO/IMAGE')
  > p.save
  > p.url
  => "http://sandbox-api1-kage.kakao.com/dn/cQhBYi/bPAa8wCC8km/bkxZTtymoZA1KKyydKibzk/img.gif"
  > p.url(:medium)
  => "http://sandbox-api1-kage.kakao.com/dn/cQhBYi/bPAa8wCC8km/bkxZTtymoZA1KKyydKibzk/img.gif?convert=320x*"
```

## Contributing

1. Fork it ( http://github.com/ultracaption/kage-ruby/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request
